#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

import os
import autoqa.util
from autotest_lib.client.bin import utils
from autoqa.test import AutoQATest
from autoqa.decorators import ExceptionCatcher

class conflicts(AutoQATest):
    version = 1

    def writelog(self, logname, logdata):
        fullname = os.path.join(self.resultsdir, logname)
        f = open(fullname, "wb")
        f.write(logdata)
        f.close()

    @ExceptionCatcher("self.run_once_failed")
    def run_once(self, baseurl, parents, name, **kwargs):
        if name:
            name = "%s-%s" % (name, autoqa.util.get_basearch())
        else:
            name = baseurl
        os.chdir(self.bindir)
        cmd = './potential_conflict.py --tempcache --newest'
        cmd += ' --repofrompath=target,%s --repoid=target' % baseurl
        count = 1
        for parent in parents:
            name = 'parent-%u' % count
            cmd += ' --repofrompath=%s,%s --repoid=%s' % (name, parent, name)
            count += 1
        # TODO set cachedir etc. to test.scratchdir?
        out = utils.system_output(cmd, retain_output=True)

        file_conflicts = []
        conflicting_pkgs = []
        hitmarker = False
        for line in out.split('\n'):
            if hitmarker:
                line.strip()
                if line.startswith('  /'):
                    file_conflicts.append(line[2:])
                elif line != '':
                    conflicting_pkgs.append(line)
            elif line.startswith('== File conflicts'):
                hitmarker = True

        self.writelog("file_conflicts", "\n".join(sorted(file_conflicts)))
        self.writelog("conflicting_pkgs", "\n".join(sorted(conflicting_pkgs)))
        self.write_test_keyval({'file_conflicts_count':len(file_conflicts),
                                'conflicting_pkgs_count':len(conflicting_pkgs)})

        if len(conflicting_pkgs) > 0:
            self.result = "PASSED"
        else:
            self.result = "FAILED"

        self.summary = "%u packages with file conflicts in %s" % (len(conflicting_pkgs), name)
        self.highlights = "= Conflicting pkgs =\n%s" % "\n".join(sorted(conflicting_pkgs))
        self.highlights += "\n\n= File conflicts =\n%s" % "\n".jouin(sorted(file_conflicts))
        self.outputs = out

